﻿using System;
using System.Globalization;
using System.Text;

namespace Library
{
    /// <summary>
    /// Az emulator memóriája
    /// </summary>
    public class EmulatorMemory
    {
        /// <summary>
        /// Kirajzolja a display-t
        /// </summary>
        public void DisplayDisplay()
        {
            for (int i = 0; i < 10; i++)
            {
                StringBuilder sb = new StringBuilder();
                for (int j = 0; j < 10; j++)
                {
                    sb.Append((char)Memory[i * 10 + j]);
                }
                Console.WriteLine(sb.ToString());
            }
        }

        /// <summary>
        /// Az aktuálisan olvasandó bájt címe
        /// </summary>
        public int CurrentIndex { get; set; } = 99;

        /// <summary>
        /// A memória
        /// </summary>
        public byte[] Memory { get; } = new byte[1024];

        /// <summary>
        /// Legutolsó MOV, ADD és SUB utasítások eredményének tárolására
        /// </summary>
        public byte LastResult { get; set; }

        /// <summary>
        /// Overflow ellenőrzés
        /// </summary>
        public bool LastOperationOverflow { get; set; }

        /// <summary>
        /// Le van-e fagyva az emulator?
        /// </summary>
        public bool Frozen { get; set; }


        /// <summary>
        /// Adott operandus által jelzett címen lévő értéket adja vissza
        /// </summary>
        /// <param name="index">A memóriacímet reprezentáló operandus</param>
        /// <returns>A memóriában az adott címen lévő érték</returns>
        public byte this[MemoryAddressOperand index]
        {
            get => Memory[index.Address];
            set => Memory[index.Address] = value;
        }

        /// <summary>
        /// Adott címen lévő értéket adja vissza
        /// </summary>
        /// <param name="index">A memóriacímet</param>
        /// <returns>A memóriában az adott címen lévő érték</returns>
        public byte this[int index]
        {
            get => Memory[index];
        }

        /// <summary>
        /// Betölti a bájtkódot a memóriába
        /// </summary>
        /// <param name="program"></param>
        public void LoadProgram(string program)
        {
            int i = 0;
            for (; i < 100; i++) // Display memória
            {
                Memory[i] = 0x20;
            }
            while (!string.IsNullOrEmpty(program)) 
            {
                Memory[i++] = byte.Parse(program.Substring(0, 2), NumberStyles.HexNumber, CultureInfo.InvariantCulture); //2-esével olvassuk a karaktereket és átalakítjuk bájttá
                program = program.Remove(0, 2);
            }
        }
    }
}
