﻿namespace Library
{
    /// <summary>
    /// Egy operátort reprezentál a programozási nyelvből
    /// </summary>
    public abstract class Operator
    {
        private readonly byte opCode;

        /// <summary>
        /// Létrehozza az operátort
        /// </summary>
        /// <param name="opCode">Az operátor bájtkódja</param>
        public Operator(byte opCode)
        {
            this.opCode = opCode;
        }

        /// <summary>
        /// Lefordítja az operátort (bájtkód+paraméterek)
        /// </summary>
        /// <returns>Az adott operátor bájtkódja stringként a paraméterekkel együtt</returns>
        public string Compile() => $"{opCode.ToString("X2")}{WriteParameters()}";

        /// <summary>
        /// Paraméterek kiírása
        /// </summary>
        /// <returns>Visszaadja a paramétereket a bájtkódba írással</returns>
        protected abstract string WriteParameters();

        /// <summary>
        /// Végrehajtja a műveletet a memórián
        /// </summary>
        /// <param name="emulatorMemory">A memóriát tartalmazó objektumpéldány</param>
        public abstract void Execute(EmulatorMemory emulatorMemory);
    }
}
