﻿using Library;
using System;
using System.Collections.Generic;
using System.IO;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace Emulator
{
    /// <summary>
    /// Az emulatort repreneztálja
    /// </summary>
    public class Emulator
    {
        private EmulatorMemory memory;
        private string program;

        // Ellenőrzi, hogy van-e program betöltve
        private bool IsProgramNotLoaded()
        {
            return memory != null;
        }

        /// <summary>
        /// Betölti a fájlban megadott bájtkódot a memóriába
        /// </summary>
        /// <param name="inputFile">A bájtkdot tartalmazó fájl elérési útvonala</param>
        public void Load(string inputFile)
        {
            memory = new EmulatorMemory();
            program = File.ReadAllText(inputFile);
            memory.LoadProgram(program);
        }

        /// <summary>
        /// Kirajtolja a display-t
        /// </summary>
        public void DisplayDisplay()
        {
            memory?.DisplayDisplay();
        }

        /// <summary>
        /// Végigfuttatja a programot
        /// </summary>
        public void Run()
        {
            if (!IsProgramNotLoaded())
            {
                Console.WriteLine("No program loaded");
                return;
            }
            if (memory.Frozen)
            {
                Console.WriteLine("Emulator is frozen");
                return;
            }
            if (memory.CurrentIndex > 1023)
            {
                Console.WriteLine("Program ended");
                return;
            }
            while (memory.CurrentIndex < 1024)
            {
                StepProgram();                
            }
        }

        /// <summary>
        /// Egyet léptet a programon
        /// </summary>
        public void Step()
        {
            if (!IsProgramNotLoaded())
            {
                Console.WriteLine("No program loaded");
                return;
            }
            if (memory.Frozen)
            {
                Console.WriteLine("Emulator is frozen");
                return;
            }
            if (memory.CurrentIndex>1023)
            {
                Console.WriteLine("Program ended");
                return;
            }
            StepProgram();            
        }

        /// <summary>
        /// Adott memóriacímen lévő értéket jelenít meg
        /// </summary>
        /// <param name="address">A memóriacím</param>
        public void Memory(int address)
        {
            if (!IsProgramNotLoaded())
            {
                Console.WriteLine("No program loaded");
                return;
            }
            if (memory.Frozen)
            {
                Console.WriteLine("Emulator is frozen");
                return;
            }
            if (address > 1023)
            {
                Console.WriteLine("Invalid address");
            }
            Console.WriteLine(this.memory[address].ToString("X2"));
        }

        // Elvégzi a program következő utasítását
        private void StepProgram()
        {
            memory.CurrentIndex++; // léptet egyet
            byte opCode = memory[memory.CurrentIndex]; // opCode beolvasása
            switch (opCode)
            {
                // Ezután minden egyes opcode-hoz beolvassuk a megfelelő mennyisű bájtot operandusként és elvégezzük a futtatást
                case 0:
                    var movop1 = 256 * memory[memory.CurrentIndex + 1] + memory[memory.CurrentIndex + 2];
                    var movop2 = 256 * memory[memory.CurrentIndex + 3] + memory[memory.CurrentIndex + 4];
                    memory.CurrentIndex = memory.CurrentIndex + 4;
                    new MovOperator { Op1 = new MemoryAddressOperand(movop1), Op2 = new MemoryAddressOperand(movop2) }.Execute(memory);
                    break;
                case 1:
                    var movcop1 = 256 * memory[memory.CurrentIndex + 1] + memory[memory.CurrentIndex + 2];
                    var movcop2 = memory[memory.CurrentIndex + 3];
                    memory.CurrentIndex = memory.CurrentIndex + 3;
                    new MovCOperator { Op1 = new MemoryAddressOperand(movcop1), Op2 = new ConstantOperand(movcop2) }.Execute(memory);
                    break;
                case 2:
                    var addop1 = 256 * memory[memory.CurrentIndex + 1] + memory[memory.CurrentIndex + 2];
                    var addop2 = 256 * memory[memory.CurrentIndex + 3] + memory[memory.CurrentIndex + 4];
                    memory.CurrentIndex = memory.CurrentIndex + 4;
                    new AddOperator { Op1 = new MemoryAddressOperand(addop1), Op2 = new MemoryAddressOperand(addop2) }.Execute(memory);
                    break;
                case 3:
                    var addcop1 = 256 * memory[memory.CurrentIndex + 1] + memory[memory.CurrentIndex + 2];
                    var addcop2 = memory[memory.CurrentIndex + 3];
                    memory.CurrentIndex = memory.CurrentIndex + 3;
                    new AddCOperator { Op1 = new MemoryAddressOperand(addcop1), Op2 = new ConstantOperand(addcop2) }.Execute(memory);
                    break;
                case 4:
                    var subop1 = 256 * memory[memory.CurrentIndex + 1] + memory[memory.CurrentIndex + 2];
                    var subop2 = 256 * memory[memory.CurrentIndex + 3] + memory[memory.CurrentIndex + 4];
                    memory.CurrentIndex = memory.CurrentIndex + 4;
                    new SubOperator { Op1 = new MemoryAddressOperand(subop1), Op2 = new MemoryAddressOperand(subop2) }.Execute(memory);
                    break;
                case 5:
                    var subcop1 = 256 * memory[memory.CurrentIndex + 1] + memory[memory.CurrentIndex + 2];
                    var subcop2 = memory[memory.CurrentIndex + 3];
                    memory.CurrentIndex = memory.CurrentIndex + 3;
                    new SubCOperator { Op1 = new MemoryAddressOperand(subcop1), Op2 = new ConstantOperand(subcop2) }.Execute(memory);
                    break;
                case 6:
                    var jmpop = 256 * memory[memory.CurrentIndex + 1] + memory[memory.CurrentIndex + 2];
                    memory.CurrentIndex = memory.CurrentIndex + 2;
                    new JmpOperator { Operand = new MemoryAddressOperand(jmpop) }.Execute(memory);
                    break;
                case 7:
                    var jzop = 256 * memory[memory.CurrentIndex + 1] + memory[memory.CurrentIndex + 2];
                    memory.CurrentIndex = memory.CurrentIndex + 2;
                    new JzOperator { Operand = new MemoryAddressOperand(jzop) }.Execute(memory);
                    break;
                case 8:
                    var jcop = 256 * memory[memory.CurrentIndex + 1] + memory[memory.CurrentIndex + 2];
                    memory.CurrentIndex = memory.CurrentIndex + 2;
                    new JcOperator { Operand = new MemoryAddressOperand(jcop) }.Execute(memory);
                    break;
                case 9:
                    new WaitOperator().Execute(memory);
                    break;
                case 10:
                    new EndOperator().Execute(memory);
                    break;
                default:
                    memory.Frozen = true;
                    break;
            }
        }
    }
}
